#!/bin/bash
## make_dist.sh for forum82 in /home/freddy/f82
##
## Made by (Freddy)
## Login   <freddi@ifrance.com>
##
## Started on  Fri Apr 16 21:38:27 2004 Freddy
##

#########################################################################
## Display functions ####################################################
#########################################################################

DEF="[0m"
BLACK="[1;30m"
RED="[1;31m"
GREEN="[1;32m"
YELLOW="[1;33m"
BLUE="[1;34m"
PINK="[1;35m"
CYAN="[1;36m"
WHITE="[1;37m"

## util function for pretty printing
## The second (optional) parameter is a color: if arg is given, combinaison of
## |.*| are colored with the given color, and length of control characters are
## not taken into account
## In that case, there MUST be EXACTLY TWO pipe, otherwise it will not function
## properly
__action_len=0
start_action()
{
  local color
  local str=$1
  local disp=$1

  if [ $# -eq 2 ]; then
    color=$2
    ## Remove the | for length computing
    str=${str//|}
    ## and replace them with color information for display
    disp=${disp/|/$color}
    disp=${disp//|/$DEF}
  fi
  echo -n "$disp"
  __action_len=${#str}
}

end_action()
{
  # print spaces to justify text
  len=$(( 55 - $__action_len))
  i=0
  while [ $i -lt $len ]; do
    echo -n " "
    i=$(( $i + 1 ))
  done
  # determine result
  case "$2" in
    warning) color=$YELLOW; res="$1";;
    success) color=$GREEN; res="$1";;
    error) color=$RED; res="$1";;
    skip)color=$YELLOW; res="Skipping";;
    0) color=$GREEN; res="done";;
    *) color=$RED; res="error";;
  esac
  # print result
  echo  " $BLUE[$DEF $color$res$DEF $BLUE]$DEF "
}

#########################################################################
## Utils functions ######################################################
#########################################################################

# Arg1: pretty name
# Arg2: directory of f82 to handle
# Arg3: file to backup (in forum82 path)
# Arg4: backup filename
# Arg5: policy for backuped file: erase / empty
# Arg6: color of dir (for display purpose only)
# Return: 0 if the file was successfully backuped or does not exist, 1 otherwise
backup_file()
{
  # give better name to parameters
  local "name=$1"
  local "dir=$2"
  local "file_path=$3"
  local "backup_name=$4"
  local "policy=$5"
  local "color=$6"
  # begin the backup
  start_action "Backup $name file in |$dir|... " "$color"
  # check the file exists
  if [ ! -r "$dir/$file_path" ]; then
    end_action "Not found" warning
    return 0
  fi
  # check if a previous backup does not exist
  if [ -r "$dir-$backup_name" ]; then
    end_action "Existing backup found" warning
    return 0
  fi
  # backup the file
  cp -f "$dir/$file_path" "$dir-$backup_name"
  if [ $? -ne 0 ]; then
    end_action "ERROR" error
    return 1
  fi
  # remove or empty the file
  case "$policy" in
    erase) rm -f "$dir/$file_path"
    if [ $? -ne 0 ]; then
      end_action "Can't erase file" error
      exit 2
    fi;;
    empty) echo -n > "$dir/$file_path";;
    *) end_action "Bad policy name: $policy" error
    exit 2;;
  esac
  end_action done success
  return 0
}


# Arg1  pretty name
# Arg2: directory of f82 to handle
# Arg3: file to restore
# Arg4: backup filename
# Arg5: color of dir (for display purpose only)
restore_file()
{
  # give bettr name to parameters
  local "name=$1"
  local "dir=$2"
  local "file_path=$3"
  local "backup_name=$4"
  local "color=$5"
  # begin the restore
  start_action "Restoring $name file in |$dir|... " "$color"
  # check if the backup exists
  if [ ! -r "$dir-$backup_name" ]; then
    end_action "No backup found" warning
    return 0
  fi
  # restore the backup
  mv -f "$dir-$backup_name" "$dir/$file_path"
  if [ $? -ne 0 ]; then
    end_action "ERROR" error
    exit 2
  fi
  end_action done success
  return 0
}

# Make a backup of all required files
# arg1 must be the name of the directory containing a version of f82
prepare_directory()
{
  backup_file 'configuration' "$1" "forum/config.php3" "config.php3" "erase" "$2"
  if [ $? -ne 0 ]; then
    exit 2
  fi
  backup_file 'statistics' "$1" "forum/stats/stats.html" "stats.html" "empty" "$2"
  if [ $? -ne 0 ]; then
    exit 2
  fi
}

# restore a directory (ie restore the configuration file
restore_directory()
{
  restore_file 'configuration' "$1" "forum/config.php3" "config.php3" "$2"
  if [ $? -ne 0 ]; then
    exit 2
  fi
  restore_file 'statistics' "$1" "forum/stats/stats.html" "stats.html" "$2"
  if [ $? -ne 0 ]; then
    exit 2
  fi
}

#########################################################################
## Main program #########################################################
#########################################################################

## Check that program is used correctly
if [ $# -ne 1 -a $# -ne 2 ]; then
  echo "Usage: $0 old_version/ [new_version/]"
  exit 1
fi

## Check that arguments are valid
### 1) retrieve new_version directory (defaults to f82, which should be a symbolic link
###    to  the real directory)
if [ -h "$1" ]; then
  old="`readlink \"$1\"`"
else
  old="$1"
fi
if [ $# -eq 1 ]; then
  new=`readlink f82`
  echo ">>> Using last version of f82 for diff ($WHITE$new$DEF)"
else
  if [ -h "$2" ]; then
    new="`readlink \"$2\"`"
  else
    new="$2"
  fi
fi

### 2) Check that arguments are valid repertory
for dir in old new; do
  if [ ! -d "${!dir}" -o ! -r "${!dir}" ]; then
    echo "ERROR: '${!dir}' is not a directory or is not readable"
    exit 1
  fi
done

### 3) Check that compared directory are different (cause problems with backups)
if [ "$old" = "$new" ]; then
  echo "${RED}ERROR$DEF: cannot diff the same directory twice"
  exit 1
fi

## Print a summary of actions
echo ">>> Generating diff between ${WHITE}$old$DEF and ${CYAN}$new$DEF"

## start a new log
rm -f log

## prepare directories
prepare_directory "$old" "$WHITE"
prepare_directory "$new" "$CYAN"

## Generate file list and ignore list
start_action "Generating file list... "
# This is for diff who only need filename pattern
# Find are for tar who need full path for ignoring files
echo -e ".*.swp\n.svn\nCVS\n#*#\n*~" > .ignore-diff								\
&& find "$old" -name ".*.swp" -o -name ".svn" -o -regex '*/CVS/*' -o -name '#*#' -o -name '*~' > .ignore	\
&& find "$new" -name ".*.swp" -o -name ".svn" -o -regex '*/CVS/*' -o -name '#*#' -o -name '*~' >> .ignore
end_action unused $?

## creating diff
start_action "Creating diff..."
diff --exclude-from=.ignore-diff -Naur "$old" "$new" > "$old-$new.diff" 2>> log
if [ $? -le 1 ]; then
  res=0
else
  res=1
fi
end_action unused $res

## compressing diff
start_action "Creating compressed diff..."
cp "$old-$new.diff" "$old-$new.diff.tmp" 2>> log			\
&& gzip -f "$old-$new.diff"						\
&& cp "$old-$new.diff.tmp" "$old-$new.diff" 2>> log			\
&& bzip2 -f "$old-$new.diff" 2>> log					\
&& mv "$old-$new.diff.tmp" "$old-$new.diff" 2>> log
end_action unused $?

## creating tarball of new version
start_action "Making tarball..."
tar --exclude-from .ignore -cvf "$new.tar" "$new/" make_dist.sh Makefile > /dev/null 2>> log
end_action unused $?

## creating compressed tarball
start_action "Making compressed tarball..."
cp "$new.tar" "$new.tar.tmp"				\
&& gzip -f "$new.tar"					\
&& cp "$new.tar.tmp" "$new.tar" 2>> log			\
&& bzip2 -f "$new.tar"	2>> log				\
&& mv "$new.tar.tmp" "$new.tar" 2>> log > /dev/null
end_action unused $?

## creation of tarball of documentation
start_action "Making documentation tarball..."
doc_ok=1
if [ -d "doc/$new" ]; then
  tar cvf "doc$new.tar" "doc/$new/" > /dev/null 2>> log
  end_action unused $?
else
  doc_ok=0
  end_action "Not found" warning
fi

## compress documentation
start_action "Compressing documentation..."
if [ "$doc_ok" -eq 1 ]; then
  cp "doc$new.tar" "doc$new.tar.tmp"				\
  && gzip -f "doc$new.tar"					\
  && cp "doc$new.tar.tmp" "doc$new.tar" 2>> log			\
  && bzip2 -f "doc$new.tar"	2>> log				\
  && mv "doc$new.tar.tmp" "doc$new.tar" 2>> log > /dev/null
  end_action unused $?
else
  end_action unused skip
fi



## restore directories
restore_directory "$old" "$WHITE"
restore_directory "$new" "$CYAN"


## Cleaning directory
echo ">>> Cleaning directory"
rm -f .ignore .ignore-diff
